@@ -1,6 +1,7 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2019-2020, Maximilian Köhl <koehl@cs.uni-saarland.de>
+# Copyright (C) 2019-2021, Saarland University
+# Copyright (C) 2019-2021, Maximilian Köhl <koehl@cs.uni-saarland.de>
 
 from __future__ import annotations
 
@@ -27,12 +28,23 @@ Bound = t.Optional[t.Union["expressions.ValueOrExpression", "ellipsis"]]
 
 
 class Type(abc.ABC):
+    """
+    A *type*.
+    """
+
     @property
     def is_numeric(self) -> bool:
+        """
+        Indicates whether the type is numeric or not.
+        """
         return False
 
     @abc.abstractmethod
     def is_assignable_from(self, typ: Type) -> bool:
+        """
+        Checks whether values of the other type can be assigned to
+        targets of this type.
+        """
         raise NotImplementedError()
 
     def validate_in(self, scope: context.Scope) -> None:
@@ -55,6 +67,13 @@ class NumericType(Type, abc.ABC):
 
 @d.dataclass(frozen=True)
 class IntegerType(NumericType):
+    """
+    The *integer type*.
+
+    .. hint::
+        Instead of instantiating new objects of this class use :attr:`INT`.
+    """
+
     def __str__(self) -> str:
         return "types.INT"
 
@@ -66,6 +85,13 @@ class IntegerType(NumericType):
 
 @d.dataclass(frozen=True)
 class RealType(NumericType):
+    """
+    The *real type*.
+
+    .. hint::
+        Instead of instantiating new objects of this class use :attr:`REAL`.
+    """
+
     def __str__(self) -> str:
         return "types.REAL"
 
@@ -75,6 +101,13 @@ class RealType(NumericType):
 
 @d.dataclass(frozen=True)
 class BoolType(Type):
+    """
+    The *boolean type*.
+
+    .. hint::
+        Instead of instantiating new objects of this class use :attr:`BOOL`.
+    """
+
     def __str__(self) -> str:
         return "types.BOOL"
 
@@ -84,6 +117,13 @@ class BoolType(Type):
 
 @d.dataclass(frozen=True)
 class ClockType(NumericType):
+    """
+    The *clock type*.
+
+    .. hint::
+        Instead of instantiating new objects of this class use :attr:`CLOCK`.
+    """
+
     def __str__(self) -> str:
         return "types.CLOCK"
 
@@ -95,6 +135,13 @@ class ClockType(NumericType):
 
 @d.dataclass(frozen=True)
 class ContinuousType(NumericType):
+    """
+    The *continuous type*.
+
+    .. hint::
+        Instead of instantiating new objects of this class use :attr:`CONTINUOUS`.
+    """
+
     def __str__(self) -> str:
         return "types.CONTINUOUS"
 
@@ -111,6 +158,19 @@ CONTINUOUS = ContinuousType()
 
 @d.dataclass(frozen=True)
 class BoundedType(NumericType):
+    """
+    A *bounded numeric type*.
+
+    Attributes
+    ----------
+    base:
+        The numeric base type.
+    lower_bound:
+        The optional lower bound for values of the type.
+    upper_bound:
+        The optional upper bound for values of the type.
+    """
+
     base: NumericType
 
     lower_bound: t.Optional[expressions.Expression]
@@ -129,8 +189,8 @@ class BoundedType(NumericType):
 
     def validate_in(self, scope: context.Scope) -> None:
         if self.lower_bound is not None:
-            if not scope.is_constant(self.lower_bound):
-                raise InvalidBoundError("`lower_bound` has to be a constant")
+            # if not scope.is_constant(self.lower_bound):
+            #     raise InvalidBoundError("`lower_bound` has to be a constant")
             lower_bound_type = scope.get_type(self.lower_bound)
             if not self.base.is_assignable_from(lower_bound_type):
                 raise InvalidBoundError(
@@ -138,8 +198,8 @@ class BoundedType(NumericType):
                     f"assignable to base-type {self.base}"
                 )
         if self.upper_bound is not None:
-            if not scope.is_constant(self.upper_bound):
-                raise InvalidBoundError("`upper_bound` has to be a constant")
+            # if not scope.is_constant(self.upper_bound):
+            #     raise InvalidBoundError("`upper_bound` has to be a constant")
             upper_bound_type = scope.get_type(self.upper_bound)
             if not self.base.is_assignable_from(upper_bound_type):
                 raise InvalidBoundError(
@@ -159,6 +219,15 @@ class BoundedType(NumericType):
 
 @d.dataclass(frozen=True)
 class ArrayType(Type):
+    """
+    An *array type*.
+
+    Attributes
+    ----------
+    element:
+        The type of the elements of the array.
+    """
+
     element: Type
 
     def __str__(self) -> str:
@@ -171,11 +240,23 @@ class ArrayType(Type):
 
 
 def array_of(element: Type) -> ArrayType:
+    """
+    Constructs an array type with the given element type.
+    """
     return ArrayType(element)
 
 
 @d.dataclass(frozen=True)
 class SetType(Type):
+    """
+    An *set type*.
+
+    Attributes
+    ----------
+    element:
+        The type of the elements of the set.
+    """
+
     element: Type
 
     def __str__(self) -> str:
@@ -186,6 +267,9 @@ class SetType(Type):
 
 
 def set_of(element: Type) -> SetType:
+    """
+    Constructs a set type with the given element type.
+    """
     return SetType(element)
 
 
